@@ -8,42 +8,21 @@ Inheritance diagram:
    :parts: 3
 """
 
-#-----------------------------------------------------------------------------
-#  Copyright (C) 2008-2011  The IPython Development Team
-#
-#  Distributed under the terms of the BSD License.  The full license is in
-#  the file COPYING, distributed as part of this software.
-#-----------------------------------------------------------------------------
-
-#-----------------------------------------------------------------------------
-# Imports
-#-----------------------------------------------------------------------------
-
 import os
 import re
 import sys
 import textwrap
 from string import Formatter
+from pathlib import Path
 
-from IPython.external.path import path
-from IPython.testing.skipdoctest import skip_doctest_py3, skip_doctest
 from IPython.utils import py3compat
 
-#-----------------------------------------------------------------------------
-# Declarations
-#-----------------------------------------------------------------------------
-
 # datetime.strftime date format for ipython
 if sys.platform == 'win32':
     date_format = "%B %d, %Y"
 else:
     date_format = "%B %-d, %Y"
 
-
-#-----------------------------------------------------------------------------
-# Code
-#-----------------------------------------------------------------------------
-
 class LSString(str):
     """String derivative with a special access attributes.
 
@@ -52,7 +31,7 @@ class LSString(str):
         .l (or .list) : value as list (split on newlines).
         .n (or .nlstr): original value (the string itself).
         .s (or .spstr): value as whitespace-separated string.
-        .p (or .paths): list of path objects
+        .p (or .paths): list of path objects (requires path.py package)
 
     Any values which require transformations are computed only once and
     cached.
@@ -87,7 +66,7 @@ class LSString(str):
         try:
             return self.__paths
         except AttributeError:
-            self.__paths = [path(p) for p in self.split('\n') if os.path.exists(p)]
+            self.__paths = [Path(p) for p in self.split('\n') if os.path.exists(p)]
             return self.__paths
 
     p = paths = property(get_paths)
@@ -113,7 +92,7 @@ class SList(list):
     * .l (or .list) : value as list (the list itself).
     * .n (or .nlstr): value as a string, joined on newlines.
     * .s (or .spstr): value as a string, joined on spaces.
-    * .p (or .paths): list of path objects
+    * .p (or .paths): list of path objects (requires path.py package)
 
     Any values which require transformations are computed only once and
     cached."""
@@ -145,7 +124,7 @@ class SList(list):
         try:
             return self.__paths
         except AttributeError:
-            self.__paths = [path(p) for p in self if os.path.exists(p)]
+            self.__paths = [Path(p) for p in self if os.path.exists(p)]
             return self.__paths
 
     p = paths = property(get_paths)
@@ -176,7 +155,7 @@ class SList(list):
             except IndexError:
                 return ""
 
-        if isinstance(pattern, py3compat.string_types):
+        if isinstance(pattern, str):
             pred = lambda x : re.search(pattern, x, re.IGNORECASE)
         else:
             pred = pattern
@@ -243,7 +222,7 @@ class SList(list):
                 try:
                     n = int(numstr)
                 except ValueError:
-                    n = 0;
+                    n = 0
                 dsu[i][0] = n
 
 
@@ -324,8 +303,10 @@ def list_strings(arg):
         Out[9]: ['A', 'list', 'of', 'strings']
     """
 
-    if isinstance(arg, py3compat.string_types): return [arg]
-    else: return arg
+    if isinstance(arg, str):
+        return [arg]
+    else:
+        return arg
 
 
 def marquee(txt='',width=78,mark='*'):
@@ -491,6 +472,17 @@ def strip_email_quotes(text):
             text = '\n'.join([ ln[strip:] for ln in lines])
     return text
 
+def strip_ansi(source):
+    """
+    Remove ansi escape codes from text.
+    
+    Parameters
+    ----------
+    source : str
+        Source to remove the ansi from
+    """
+    return re.sub(r'\033\[(\d|;)+?m', '', source)
+
 
 class EvalFormatter(Formatter):
     """A String Formatter that allows evaluation of simple expressions.
@@ -521,7 +513,6 @@ class EvalFormatter(Formatter):
 # inside [], so EvalFormatter can handle slicing. Once we only support 3.4 and
 # above, it should be possible to remove FullEvalFormatter.
 
-@skip_doctest_py3
 class FullEvalFormatter(Formatter):
     """A String Formatter that allows evaluation of simple expressions.
     
@@ -537,19 +528,17 @@ class FullEvalFormatter(Formatter):
 
         In [1]: f = FullEvalFormatter()
         In [2]: f.format('{n//4}', n=8)
-        Out[2]: u'2'
+        Out[2]: '2'
 
         In [3]: f.format('{list(range(5))[2:4]}')
-        Out[3]: u'[2, 3]'
+        Out[3]: '[2, 3]'
 
         In [4]: f.format('{3*2}')
-        Out[4]: u'6'
+        Out[4]: '6'
     """
     # copied from Formatter._vformat with minor changes to allow eval
     # and replace the format_spec code with slicing
-    def _vformat(self, format_string, args, kwargs, used_args, recursion_depth):
-        if recursion_depth < 0:
-            raise ValueError('Max string recursion exceeded')
+    def vformat(self, format_string, args, kwargs):
         result = []
         for literal_text, field_name, format_spec, conversion in \
                 self.parse(format_string):
@@ -577,10 +566,9 @@ class FullEvalFormatter(Formatter):
                 # format the object and append to the result
                 result.append(self.format_field(obj, ''))
 
-        return u''.join(py3compat.cast_unicode(s) for s in result)
+        return ''.join(py3compat.cast_unicode(s) for s in result)
 
 
-@skip_doctest_py3
 class DollarFormatter(FullEvalFormatter):
     """Formatter allowing Itpl style $foo replacement, for names and attribute
     access only. Standard {foo} replacement also works, and allows full
@@ -592,15 +580,15 @@ class DollarFormatter(FullEvalFormatter):
 
         In [1]: f = DollarFormatter()
         In [2]: f.format('{n//4}', n=8)
-        Out[2]: u'2'
+        Out[2]: '2'
 
         In [3]: f.format('23 * 76 is $result', result=23*76)
-        Out[3]: u'23 * 76 is 1748'
+        Out[3]: '23 * 76 is 1748'
 
         In [4]: f.format('$a or {b}', a=1, b=2)
-        Out[4]: u'1 or 2'
+        Out[4]: '1 or 2'
     """
-    _dollar_pattern = re.compile("(.*?)\$(\$?[\w\.]+)")
+    _dollar_pattern_ignore_single_quote = re.compile(r"(.*?)\$(\$?[\w\.]+)(?=([^']*'[^']*')*[^']*$)")
     def parse(self, fmt_string):
         for literal_txt, field_name, format_spec, conversion \
                     in Formatter.parse(self, fmt_string):
@@ -608,7 +596,7 @@ class DollarFormatter(FullEvalFormatter):
             # Find $foo patterns in the literal text.
             continue_from = 0
             txt = ""
-            for m in self._dollar_pattern.finditer(literal_txt):
+            for m in self._dollar_pattern_ignore_single_quote.finditer(literal_txt):
                 new_txt, new_field = m.group(1,2)
                 # $$foo --> $foo
                 if new_field.startswith("$"):
@@ -625,25 +613,30 @@ class DollarFormatter(FullEvalFormatter):
 # Utils to columnize a list of string
 #-----------------------------------------------------------------------------
 
-def _chunks(l, n):
-    """Yield successive n-sized chunks from l."""
-    for i in py3compat.xrange(0, len(l), n):
-        yield l[i:i+n]
+def _col_chunks(l, max_rows, row_first=False):
+    """Yield successive max_rows-sized column chunks from l."""
+    if row_first:
+        ncols = (len(l) // max_rows) + (len(l) % max_rows > 0)
+        for i in range(ncols):
+            yield [l[j] for j in range(i, len(l), ncols)]
+    else:
+        for i in range(0, len(l), max_rows):
+            yield l[i:(i + max_rows)]
 
 
-def _find_optimal(rlist , separator_size=2 , displaywidth=80):
+def _find_optimal(rlist, row_first=False, separator_size=2, displaywidth=80):
     """Calculate optimal info to columnize a list of string"""
-    for nrow in range(1, len(rlist)+1) :
-        chk = list(map(max,_chunks(rlist, nrow)))
-        sumlength = sum(chk)
-        ncols = len(chk)
-        if sumlength+separator_size*(ncols-1) <= displaywidth :
-            break;
-    return {'columns_numbers' : ncols,
-            'optimal_separator_width':(displaywidth - sumlength)/(ncols-1) if (ncols -1) else 0,
-            'rows_numbers' : nrow,
-            'columns_width' : chk
-           }
+    for max_rows in range(1, len(rlist) + 1):
+        col_widths = list(map(max, _col_chunks(rlist, max_rows, row_first)))
+        sumlength = sum(col_widths)
+        ncols = len(col_widths)
+        if sumlength + separator_size * (ncols - 1) <= displaywidth:
+            break
+    return {'num_columns': ncols,
+            'optimal_separator_width': (displaywidth - sumlength) // (ncols - 1) if (ncols - 1) else 0,
+            'max_rows': max_rows,
+            'column_widths': col_widths
+            }
 
 
 def _get_or_default(mylist, i, default=None):
@@ -654,8 +647,7 @@ def _get_or_default(mylist, i, default=None):
         return mylist[i]
 
 
-@skip_doctest
-def compute_item_matrix(items, empty=None, *args, **kwargs) :
+def compute_item_matrix(items, row_first=False, empty=None, *args, **kwargs) :
     """Returns a nested list, and info to columnize items
 
     Parameters
@@ -663,12 +655,15 @@ def compute_item_matrix(items, empty=None, *args, **kwargs) :
 
     items
         list of strings to columize
+    row_first : (default False)
+        Whether to compute columns for a row-first matrix instead of
+        column-first (default).
     empty : (default None)
         default value to fill list if needed
     separator_size : int (default=2)
-        How much caracters will be used as a separation between each columns.
+        How much characters will be used as a separation between each columns.
     displaywidth : int (default=80)
-        The width of the area onto wich the columns should enter
+        The width of the area onto which the columns should enter
 
     Returns
     -------
@@ -676,18 +671,18 @@ def compute_item_matrix(items, empty=None, *args, **kwargs) :
     strings_matrix
 
         nested list of string, the outer most list contains as many list as
-        rows, the innermost lists have each as many element as colums. If the
+        rows, the innermost lists have each as many element as columns. If the
         total number of elements in `items` does not equal the product of
         rows*columns, the last element of some lists are filled with `None`.
 
     dict_info
         some info to make columnize easier:
 
-        columns_numbers
+        num_columns
           number of columns
-        rows_numbers
-          number of rows
-        columns_width
+        max_rows
+          maximum number of rows (final number may be less)
+        column_widths
           list of with of each columns
         optimal_separator_width
           best separator width between columns
@@ -697,24 +692,22 @@ def compute_item_matrix(items, empty=None, *args, **kwargs) :
     ::
 
         In [1]: l = ['aaa','b','cc','d','eeeee','f','g','h','i','j','k','l']
-           ...: compute_item_matrix(l,displaywidth=12)
-        Out[1]:
-            ([['aaa', 'f', 'k'],
-            ['b', 'g', 'l'],
-            ['cc', 'h', None],
-            ['d', 'i', None],
-            ['eeeee', 'j', None]],
-            {'columns_numbers': 3,
-            'columns_width': [5, 1, 1],
-            'optimal_separator_width': 2,
-            'rows_numbers': 5})
+        In [2]: list, info = compute_item_matrix(l, displaywidth=12)
+        In [3]: list
+        Out[3]: [['aaa', 'f', 'k'], ['b', 'g', 'l'], ['cc', 'h', None], ['d', 'i', None], ['eeeee', 'j', None]]
+        In [4]: ideal = {'num_columns': 3, 'column_widths': [5, 1, 1], 'optimal_separator_width': 2, 'max_rows': 5}
+        In [5]: all((info[k] == ideal[k] for k in ideal.keys()))
+        Out[5]: True
     """
-    info = _find_optimal(list(map(len, items)), *args, **kwargs)
-    nrow, ncol = info['rows_numbers'], info['columns_numbers']
-    return ([[ _get_or_default(items, c*nrow+i, default=empty) for c in range(ncol) ] for i in range(nrow) ], info)
+    info = _find_optimal(list(map(len, items)), row_first, *args, **kwargs)
+    nrow, ncol = info['max_rows'], info['num_columns']
+    if row_first:
+        return ([[_get_or_default(items, r * ncol + c, default=empty) for c in range(ncol)] for r in range(nrow)], info)
+    else:
+        return ([[_get_or_default(items, c * nrow + r, default=empty) for c in range(ncol)] for r in range(nrow)], info)
 
 
-def columnize(items, separator='  ', displaywidth=80):
+def columnize(items, row_first=False, separator='  ', displaywidth=80, spread=False):
     """ Transform a list of strings into a single string with columns.
 
     Parameters
@@ -722,6 +715,10 @@ def columnize(items, separator='  ', displaywidth=80):
     items : sequence of strings
         The strings to process.
 
+    row_first : (default False)
+        Whether to compute columns for a row-first matrix instead of
+        column-first (default).
+
     separator : str, optional [default is two spaces]
         The string that separates columns.
 
@@ -732,11 +729,13 @@ def columnize(items, separator='  ', displaywidth=80):
     -------
     The formatted string.
     """
-    if not items :
+    if not items:
         return '\n'
-    matrix, info = compute_item_matrix(items, separator_size=len(separator), displaywidth=displaywidth)
+    matrix, info = compute_item_matrix(items, row_first=row_first, separator_size=len(separator), displaywidth=displaywidth)
+    if spread:
+        separator = separator.ljust(int(info['optimal_separator_width']))
     fmatrix = [filter(None, x) for x in matrix]
-    sjoin = lambda x : separator.join([ y.ljust(w, ' ') for y, w in zip(x, info['columns_width'])])
+    sjoin = lambda x : separator.join([ y.ljust(w, ' ') for y, w in zip(x, info['column_widths'])])
     return '\n'.join(map(sjoin, fmatrix))+'\n'
 
 
@@ -770,4 +769,4 @@ def get_text_list(list_, last_sep=' and ', sep=", ", wrap_item_with=""):
         return list_[0]
     return '%s%s%s' % (
         sep.join(i for i in list_[:-1]),
-        last_sep, list_[-1])
\ No newline at end of file
+        last_sep, list_[-1])
